/*
 * darcnes_app.cpp
 *
 * DarcNES/BeOS main application class
 */

/* $Id: darcnes_app.cpp,v 1.1 2000/05/29 23:54:49 nyef Exp $ */

#include "darcnes_app.h"

#include <Application.h>
#include <Window.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <FilePanel.h>
#include <stdio.h>
#include <string.h>

extern "C" {
#include "tool.h"
#include "system.h"
}

#include "darcnes_window.h"
#include "emulation.h"

darcnes_app::darcnes_app()
    : BApplication("application/x-vnd.Nyef-DarcNES")
{
    BRect window_frame;

    system_type = ST_NONE;
    romfile = NULL;

    window_frame.Set(50, 50, 200, 200);
    main_window = new darcnes_window(window_frame);
}

void darcnes_app::ArgvReceived(int32 argc, char *argv[])
{
    int i;

    for (i = 1; i < argc; i++) {
	if (!strncasecmp(argv[i], "--system=", 9)) {
	    system_type = parse_system_name(argv[i] + 9);
	} else if (romfile) {
	    printf("rom file \"%s\" already loaded, ignoring \"%s\"\n", romfile->filename, argv[i]);
	} else if ((romfile = read_romimage(argv[i]))) {
	    if (system_type == ST_NONE) {
		system_type = guess_system(romfile);
	    }
	} else {
	    printf("error loading rom file \"%s\"\n", argv[i]);
	}
    }

    if (romfile) {
	main_window->set_emulation(new emulation(system_type, romfile));
    }
}

void darcnes_app::RefsReceived(BMessage *message)
{
    entry_ref ref;
    status_t error;

    error = message->FindRef("refs", 0, &ref);
    if (error != B_OK) {
	return;
    }

    romfile = read_romimage(ref.name);
    if (system_type == ST_NONE) {
	system_type = guess_system(romfile);
    }

    if (romfile) {
	main_window->set_emulation(new emulation(system_type, romfile));
    }
}


/*
 * $Log: darcnes_app.cpp,v $
 * Revision 1.1  2000/05/29 23:54:49  nyef
 * Initial revision
 *
 */
