/*
 * darcnes_window.cpp
 *
 * DarcNES emulation window
 */

/* $Id: darcnes_window.cpp,v 1.1 2000/05/29 23:55:00 nyef Exp $ */

#include "darcnes_window.h"

#include <Application.h>
#include <Window.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <FilePanel.h>

extern "C" {
#include "types.h"
}

#include "emulation.h"

const uint32 MENU_FILE_OPEN  = 'MFop';
const uint32 MENU_FILE_START = 'MFst';

darcnes_window::darcnes_window(BRect frame)
    : BWindow(frame, "DarcNES/BeOS", B_TITLED_WINDOW,
	      B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
    extern void ui_initialize(darcnes_window *window); /* FIXME: Wrong! */
    BRect r;

    r = Bounds();

    r.top = 20;
    game_display = new BView(r, NULL, B_FOLLOW_TOP | B_FOLLOW_LEFT, 0);
    AddChild(game_display);

    r.top = 0;
    r.bottom = 19;
    menubar = new BMenuBar(r, "menu_bar");
    AddChild(menubar);
    populate_menubar();

    open_panel = new BFilePanel;

    image_bitmap = NULL;
    the_emulation = NULL;

    ui_initialize(this);

    Show();
}

void darcnes_window::populate_menubar()
{
    BMenu *menu;

    menu = new BMenu("File");
    file_open = new BMenuItem("Open" B_UTF8_ELLIPSIS, new BMessage(MENU_FILE_OPEN), 'O');
    menu->AddItem(file_open);
    menu->AddSeparatorItem();
    menu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q'));
    menubar->AddItem(menu);
}

bool darcnes_window::QuitRequested()
{
    if (the_emulation) {
	the_emulation->stop();
    }

    be_app->PostMessage(B_QUIT_REQUESTED);
    return true;
}

void darcnes_window::MessageReceived(BMessage *message)
{
    extern void handle_joypad_key(int is_press, int32 key); /* FIXME: Wrong! */
    int32 key;

    message->FindInt32("key", &key);
    switch (message->what) {
    case MENU_FILE_OPEN:
	open_panel->Show();
	break;

    case MENU_FILE_START:
	if (the_emulation) {
	    the_emulation->start();
	}
	break;

    case B_KEY_DOWN:
	handle_joypad_key(1, key);
	break;
	
    case B_KEY_UP:
	handle_joypad_key(0, key);
	break;

    default:
	BWindow::MessageReceived(message);
	break;
    }
}

void darcnes_window::set_emulation(emulation *the_emulation)
{
    this->the_emulation = the_emulation;
    file_open->SetEnabled(false);
    the_emulation->start();
}

void darcnes_window::display_buffer()
{
    Lock();
    game_display->DrawBitmap(image_bitmap, B_ORIGIN);
    Unlock();
}

void darcnes_window::setsize(int x, int y)
{
    extern u8 *image_buffer; /* FIXME: Wrong! */
    extern int image_bytes_per_line; /* FIXME: Wrong! */

    if (image_bitmap) {
	delete image_bitmap;
    }

    Lock();
    ResizeTo(x - 1, y + 19);
    game_display->ResizeTo(x, y);
    Unlock();

    image_bitmap = new BBitmap(BRect(0, 0, x - 1, y - 1), B_CMAP8);

    image_buffer = (u8 *)image_bitmap->Bits();
    image_bytes_per_line = image_bitmap->BytesPerRow();
}


/*
 * $Log: darcnes_window.cpp,v $
 * Revision 1.1  2000/05/29 23:55:00  nyef
 * Initial revision
 *
 */
