/*
 * ui_beos.cpp
 *
 * BeOS user interface code
 */

/* $Id: ui_beos.cpp,v 1.1 2000/05/29 23:55:04 nyef Exp $ */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include <Bitmap.h>
#include <Screen.h>
#include <View.h>

extern "C" {
#include "ui.h"
#include "video.h"
#include "types.h"
#include "system.h"
}

#include "darcnes_window.h"

/* the joypad (only one, I could care less about more) */
struct joypad *ui_joypad;

/* the keypad (only one, I could care less about more) */
struct keypad *ui_keypad;

unsigned char *vid_pre_xlat;



/* debug console handling */

void deb_printf(const char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    vprintf(fmt, args);
    va_end(args);
}


/* video code */

darcnes_window *ui_window;

u8 *image_buffer;
int image_bytes_per_line;

u8 *video_get_vbp(int scanline)
{
    return &image_buffer[scanline * image_bytes_per_line];
}

void video_setsize(int x, int y)
{
    ui_window->setsize(x, y);
}

void video_display_buffer(void)
{
    ui_window->display_buffer();
}

void video_setpal(int colors, int *red, int *green, int *blue)
{
    BScreen screen;
    int i;
    
    vid_pre_xlat = (u8 *)malloc(colors);
    
    for (i = 0; i < colors; i++) {
	vid_pre_xlat[i] = screen.IndexForColor(red[i], green[i], blue[i]);
    }
}

void ui_initialize(darcnes_window *window)
{
    ui_window = window;
}


/* keypad emulation */

int keypad_register(struct keypad *pad)
{
    if (ui_keypad) {
	return 0;
    }
    
    ui_keypad = pad;
    return 1;
}

void keypad_update(struct keypad *pad)
{
    /* NOTE: does nothing. may want to do stuff when using real keypads */
}


/* joypad emulation */

int ui_register_joypad(struct joypad *pad)
{
    if (ui_joypad) {
	return 0;
    }
    
    ui_joypad = pad;
    return 1;
}

void ui_update_joypad(struct joypad *pad)
{
    /* NOTE: does nothing. may want to do stuff when using real gamepads */
}

char joypad_key_state[12];

struct {
    int32 key;
    int state;
} joypad_keys[] = {
    {0x3d, 8}, /* {XK_S, 8}, */
    {0x3c, 9}, /* {XK_A, 9}, */
    {0x31, 10}, /* {XK_braceleft, 10}, */
    {0x32, 11}, /* {XK_braceright, 11}, */
    {0x57, 0}, {0x38, 4}, /* {XK_KP_Up, 4}, */
    {0x62, 1}, {0x59, 5}, /* {XK_KP_Down, 5}, */
    {0x61, 2}, {0x48, 6}, /* {XK_KP_Left, 6}, */
    {0x63, 3}, {0x4a, 7}, /* {XK_KP_Right, 7}, */
};

void handle_joypad_key(int is_press, int32 key)
{
    int i;

    for (i = (sizeof(joypad_keys) / sizeof(joypad_keys[0])) - 1; i >= 0; i--) {
	if (key == joypad_keys[i].key) {
	    if (is_press) {
		joypad_key_state[joypad_keys[i].state] = 1;
	    } else {
		joypad_key_state[joypad_keys[i].state] = 0;
	    }
	}
    }
    
    if (!ui_joypad) {
	return;
    }

    switch (ui_joypad->button_template->num_buttons) {
	/* NOTE: all cases fall through */
    case 8:
	if (joypad_key_state[11]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[7];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[7];
	}
	
    case 7:
	if (joypad_key_state[10]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[6];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[6];
	}
	
    case 6:
	if (joypad_key_state[9]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[5];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[5];
	}
	
    case 5:
	if (joypad_key_state[8]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[4];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[4];
	}
	
    case 4:
	if (joypad_key_state[7] || joypad_key_state[3]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[3];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[3];
	}
	
    case 3:
	if (joypad_key_state[6] || joypad_key_state[2]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[2];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[2];
	}
	
    case 2:
	if (joypad_key_state[5] || joypad_key_state[1]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[1];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[1];
	}
	
    case 1:
	if (joypad_key_state[4] || joypad_key_state[0]) {
	    ui_joypad->data |= ui_joypad->button_template->buttons[0];
	} else {
	    ui_joypad->data &= ~ui_joypad->button_template->buttons[0];
	}
	
    default:
	;
    }
}


/* keyboard emulation */

extern "C" {
    /* FIXME: Should be in a header somewhere */
    void kb_init(void);
}

void kb_init(void)
{
}


/*
 * $Log: ui_beos.cpp,v $
 * Revision 1.1  2000/05/29 23:55:04  nyef
 * Initial revision
 *
 */
